--------------------------------------------------------------------------------
-- Company: <Name>
--
-- File: apb_slave.vhd
-- File history:
--      <Revision number>: <Date>: <Comments>
--      <Revision number>: <Date>: <Comments>
--      <Revision number>: <Date>: <Comments>
--
-- Description: 
--
-- <Description here>
--
-- Targeted device: <Family::PolarFire> <Die::MPF300T_ES> <Package::FCG1152>
-- Author: <Name>
--
--------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity apb_slave is
    port(
        PCLK            : in  std_logic;                        -- APB clock
        PRESETn         : in  std_logic;                        -- APB reset
        PENABLE         : in  std_logic;                        -- APB enable
        PSEL            : in  std_logic;                        -- APB periph select
        PADDR           : in  std_logic_vector(5 downto 2);     -- APB address bus
        PWRITE          : in  std_logic;                        -- APB write
        PWDATA          : in  std_logic_vector(31 downto 0);    -- APB write data
        PRDATA          : out std_logic_vector(31 downto 0);    -- APB read data
        PREADY          : out std_logic;                        -- APB ready signal data
        PSLVERR         : out std_logic;                        -- APB error signal
        DATA_OUT        : out std_logic_vector(7 downto 0);
        REG_IN_0        : in  std_logic_vector(15 downto 0);
        REG_IN_1        : in  std_logic_vector(15 downto 0);
        REG_IN_2        : in  std_logic_vector(15 downto 0);
        REG_IN_3        : in  std_logic_vector(15 downto 0);
        REG_IN_4        : in  std_logic_vector(15 downto 0);
        REG_IN_5        : in  std_logic_vector(15 downto 0);
        REG_IN_6        : in  std_logic_vector(15 downto 0);
        REG_IN_7        : in  std_logic_vector(15 downto 0);
        start           : out std_logic
	);
end apb_slave;

architecture architecture_apb_slave of apb_slave is

    signal CoreWrEn   : std_logic;
    signal CoreRdEn   : std_logic;
	signal Register_0 : std_logic_vector(31 downto 0);
	signal Register_1 : std_logic_vector(31 downto 0);
	signal Register_2 : std_logic_vector(31 downto 0);
	signal Register_3 : std_logic_vector(31 downto 0);
	signal Register_4 : std_logic_vector(31 downto 0);
	signal Register_5 : std_logic_vector(31 downto 0);
	signal Register_6 : std_logic_vector(31 downto 0);
	signal Register_7 : std_logic_vector(31 downto 0);
	signal Register_8 : std_logic_vector(31 downto 0);
	signal Register_9 : std_logic_vector(31 downto 0);
	signal Register_10 : std_logic_vector(31 downto 0);
	signal Register_11 : std_logic_vector(31 downto 0);

    signal start_i : std_logic;
begin

 	PREADY  <= '1'; 
	PSLVERR <= '0';

    DATA_OUT <= Register_8(7 downto 0);

    Register_0 <= X"0000" & REG_IN_0;
    Register_1 <= X"0000" & REG_IN_1;
    Register_2 <= X"0000" & REG_IN_2;
    Register_3 <= X"0000" & REG_IN_3;
    Register_4 <= X"0000" & REG_IN_4;
    Register_5 <= X"0000" & REG_IN_5;
    Register_6 <= X"0000" & REG_IN_6;
    Register_7 <= X"0000" & REG_IN_7;
    Register_9 <= X"12345678";
    Register_10<= X"CAFEAFFE";
    Register_11<= X"DEADBEEF";

    CoreWrEn <= '1' when (PSEL = '1' and PWRITE = '1' and PENABLE = '1') else '0';

    start <= start_i;

    apbwriteP : process (PCLK, PRESETn)
    begin
        if (PRESETn = '0') then
            Register_8 <= (others => '0');
            start_i    <= '0';
        elsif (PCLK'event and PCLK = '1') then
            if (CoreWrEn = '1') then
                case PADDR is
                    when  X"8"=>
                        Register_8 <= PWDATA;
                        start_i    <= not start_i;
                    when others => null;
                end case;
            end if;
        end if;
    end process apbwriteP;


    CoreRdEn <= '1' when (PSEL = '1' and PWRITE = '0') else '0';

    apbreadP : process (PCLK, PRESETn)
    begin
        if (PRESETn = '0') then
          PRDATA <= (others => '0');
        elsif (PCLK'event and PCLK = '1') then
            if (CoreRdEn = '1') then
                case PADDR is
                    when  X"0"=> PRDATA <= Register_0;
                    when  X"1"=> PRDATA <= Register_1;
                    when  X"2"=> PRDATA <= Register_2;
                    when  X"3"=> PRDATA <= Register_3;
                    when  X"4"=> PRDATA <= Register_4;
                    when  X"5"=> PRDATA <= Register_5;
                    when  X"6"=> PRDATA <= Register_6;
                    when  X"7"=> PRDATA <= Register_7;
                    when  X"8"=> PRDATA <= Register_8;
                    when  X"9"=> PRDATA <= Register_9;
                    when  X"A"=> PRDATA <= Register_10;
                    when  X"B"=> PRDATA <= Register_11;
                    when others => null;
                end case;
            end if;
        end if;
    end process apbreadP;

end architecture_apb_slave;
