--------------------------------------------------------------------------------
-- Company: <Name>
--
-- File: receive_xcvr_8b10b.vhd
-- File history:
--      <Revision number>: <Date>: <Comments>
--      <Revision number>: <Date>: <Comments>
--      <Revision number>: <Date>: <Comments>
--
-- Description: 
--
-- <Description here>
--
-- Targeted device: <Family::PolarFire> <Die::MPF300T_ES> <Package::FCG1152>
-- Author: <Name>
--
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_arith.all;
use IEEE.STD_LOGIC_UNSIGNED.all;


entity receive_xcvr_8b10b is
    
    port (clk     : in  std_logic;
          rstn    : in  std_logic;
          sysclk  : in std_logic;
          data_in : in std_logic_vector(15 downto 0);
          k_in    : in std_logic_vector(1 downto 0);

          reg_0 : out std_logic_vector(15 downto 0);
          reg_1 : out std_logic_vector(15 downto 0);
          reg_2 : out std_logic_vector(15 downto 0);
          reg_3 : out std_logic_vector(15 downto 0);
          reg_4 : out std_logic_vector(15 downto 0);
          reg_5 : out std_logic_vector(15 downto 0);
          reg_6 : out std_logic_vector(15 downto 0);
          reg_7 : out std_logic_vector(15 downto 0)
          );

end receive_xcvr_8b10b;

architecture architecture_receive_xcvr_8b10b of receive_xcvr_8b10b is

    type REC_ARRAY is array (0 to 7) of std_logic_vector(15 downto 0);
    signal rec_frame : REC_ARRAY; 
    signal copy_frame : REC_ARRAY; 
    type receive_states is (idle_state,
                            copy_state,
                            done_state
                            );
    signal rec_states : receive_states;

    signal frame_update : std_logic;
    signal update_sync : std_logic_vector(3 downto 0);
    signal k_in_del : std_logic_vector(1 downto 0);

    
begin


-- Frame empfangen
recP:process(clk,rstn)
    begin
        if(rstn = '0')then
            for i in 0 to 7 loop
                rec_frame(i) <= (others => '0');
            end loop;
            k_in_del <= "00";
        elsif(clk'event and clk = '1')then
            k_in_del <= k_in;
            rec_frame(0) <= data_in;
            rec_frame(1) <= rec_frame(0);
            rec_frame(2) <= rec_frame(1);
            rec_frame(3) <= rec_frame(2);
            rec_frame(4) <= rec_frame(3);
            rec_frame(5) <= rec_frame(4);
            rec_frame(6) <= rec_frame(5);
            rec_frame(7) <= rec_frame(6);
        end if;
    end process recP;

-- Frame kopieren -> 8 Takte Zeit zur Verarbeitung und Synchronisation
copyP:process(clk,rstn)
    begin
        if(rstn = '0')then
            for i in 0 to 7 loop
                copy_frame(i) <= (others => '0');
            end loop;
        elsif(clk'event and clk = '1')then
            if(rec_states = copy_state)then      -- and k_in = '0'
                for i in 0 to 7 loop
                    copy_frame(i) <= rec_frame(i);
                end loop;
            end if;
        end if;
    end process copyP;

    
 
-- Statemachine
rec_state_P : process(clk, rstn)
    begin
        if rstn = '0' then
            rec_states <= idle_state;
        elsif (clk'event and clk = '1') then
            case rec_states is
                when idle_state =>
                    if(k_in = "11")then
                        rec_states <= copy_state;
                    end if;
                when copy_state =>
                        rec_states <= done_state;
                when done_state =>
                    if(k_in = "00")then
                        rec_states <= idle_state;
                    end if;
                when others =>
                        rec_states <= idle_state;
                        
            end case;
        end if;
    end process rec_state_P;
    
--frame_update <= '0' when rec_states = idle_state  or rec_states = done_state else '1';
frame_update <= '1' when rec_states = done_state else '0';
    
-- Clock Domain Crossing
updateP:process(sysclk,rstn)
    begin
        if(rstn = '0')then
            update_sync <= (others => '0');
            reg_0 <= (others => '0');
            reg_1 <= (others => '0');
            reg_2 <= (others => '0');
            reg_3 <= (others => '0');
            reg_4 <= (others => '0');
            reg_5 <= (others => '0');
            reg_6 <= (others => '0');
            reg_7 <= (others => '0');
        elsif(sysclk'event and sysclk = '1')then
            update_sync <= update_sync(2 downto 0) & frame_update;
            if(update_sync(3 downto 2) = "01")then
                reg_0 <= copy_frame(0);
                reg_1 <= copy_frame(1);
                reg_2 <= copy_frame(2);
                reg_3 <= copy_frame(3);
                reg_4 <= copy_frame(4);
                reg_5 <= copy_frame(5);
                reg_6 <= copy_frame(6);
                reg_7 <= copy_frame(7);
            end if;
        end if;
    end process updateP;
  
    
end architecture_receive_xcvr_8b10b;
